// Copyright 2017 Jeff Foley. All rights reserved.
// Use of this source code is governed by Apache 2 LICENSE that can be found in the LICENSE file.

package sources

import (
	"net/url"
	"strconv"
	"time"

	"github.com/OWASP/Amass/amass/core"
	"github.com/OWASP/Amass/amass/utils"
)

// Baidu is data source object type that implements the DataSource interface.
type Baidu struct {
	BaseDataSource
	quantity int
	limit    int
}

// NewBaidu returns an initialized Baidu as a DataSource.
func NewBaidu(srv core.AmassService) DataSource {
	b := &Baidu{
		quantity: 20,
		limit:    100,
	}

	b.BaseDataSource = *NewBaseDataSource(srv, core.SCRAPE, "Baidu")
	return b
}

// Query returns the subdomain names discovered when querying this data source.
func (b *Baidu) Query(domain, sub string) []string {
	var unique []string

	if domain != sub {
		return unique
	}

	re := utils.SubdomainRegex(domain)
	num := b.limit / b.quantity
	t := time.NewTicker(time.Second)
	defer t.Stop()
loop:
	for i := 0; i < num; i++ {
		b.Service.SetActive()

		select {
		case <-b.Service.Quit():
			break loop
		case <-t.C:
			u := b.urlByPageNum(domain, i)
			page, err := utils.GetWebPage(u, nil)
			if err != nil {
				b.Service.Config().Log.Printf("%s: %v", u, err)
				break
			}

			for _, sd := range re.FindAllString(page, -1) {
				if u := utils.NewUniqueElements(unique, sd); len(u) > 0 {
					unique = append(unique, u...)
				}
			}
		}
	}
	return unique
}

func (b *Baidu) urlByPageNum(domain string, page int) string {
	pn := strconv.Itoa(page)
	u, _ := url.Parse("https://www.baidu.com/s")

	u.RawQuery = url.Values{"pn": {pn}, "wd": {domain}, "oq": {domain}}.Encode()
	return u.String()
}
